import greenfoot.*;
import java.util.*;

public class Mars extends World
{
    private SzenarioTyp aktuellesSzenario;
    private String aktuellesSzenarioString;
    private String szenarioArt;
    private String gegnertyp = "";
    private String worldBackgroundImageFilename;
    private GreenfootImage gfImageBG;
    private Color rasterfarbe = new Color(20, 20, 20, 60);
    private Peetie peetie;
    private int gridSize = 50; // Anpassen nach Belieben; Achtung, dann Peeties Groesse aendern! defaultwert = 50

    // Ballerszenarien
    private int maxAnzahlDrohnen = 7;
    private int anzahlAsteroiden = 1;
    public int getAnzahlAsteroidenGesamt() { return this.anzahlAsteroiden; }
    private List<Gegner> anzahlGegnerAufWelt = new ArrayList<Gegner>();
    private int drohnenVerzoegerung = 100;
    private int drohnenCounter = this.drohnenVerzoegerung; // Verzögerung zwischen neu auftauchenden Drohnen
    private boolean alleDrohnenSindSchonDa = false; // wann werden keine Drohnen mehr gespawnt
    private boolean alleAsteroidenSindSchonDa = false; // wann sind genug Asteroiden da?
    // private int gridSize = 50; // anpassen nach belieben; achtung, dann peeties groesse ändern! defaultwert = 50

    public Mars(SzenarioTyp s) {
        super(SzenarioKonfigurator.worldWidth, SzenarioKonfigurator.worldHeight, 1);  // Weltgröße direkt setzen
        this.aktuellesSzenario = s;
        this.aktuellesSzenarioString = this.aktuellesSzenario.toString();
        this.szenarioArt = SzenarioKonfigurator.szenarioArt;
        Sound.initializeDefaultSounds();

        this.worldBackgroundImageFilename = SzenarioKonfigurator.worldBackgroundImageFilename;
        this.gegnertyp = SzenarioKonfigurator.gegnertyp;
        // Welt initialisieren
        initializeWorld();

        // Objekte in der Welt verteilen
        SzenarioKonfigurator.objekteVerteilen(s, this);
        if(aktuellesSzenarioString.matches("e4m[5-9]|e3m5"))  { // bei den szenarien e3m5 und e4m5 bis 9 wird hintergrundmusik gespielt (Drohnenszenarien)
            Sound.play("BGMUSIC");
        } else if(aktuellesSzenarioString.equals("e7m7")) {
            Sound.play("LAUNCHCOMPLETE");
        }
        this.setPaintOrder(Peetie.class);
    }

    public void act() {
        switch(this.gegnertyp) {
            case "":
                // keine Gegner
                break;
            case "kampfdrohnen":
                this.drohnenSteuerung();
                break;
            case "scoutdrohnen":
                this.drohnenSteuerung();
                break;
            case "schrott":
                this.schrottSteuerung();
                break;
            case "asteroiden":
                this.asteroidenSteuerung();

                break;
        }

    }

    public void initializeWorld() {
        // Alle Objekte entfernen
        for(Actor a : (List<Actor>)this.getObjects(Actor.class)) {
            this.removeObject(a);
        }

        // Szenario-Variablen initialisieren
        // SzenarioKonfigurator.initialisiereSzenario(this.aktuellesSzenario);  

        // Hintergrund setzen
        this.setBackground(this.worldBackgroundImageFilename);
        this.gfImageBG = this.getBackground();
        this.peetie = new Peetie();

        // Peetie hinzufügen (an eine Position außerhalb des sichtbaren Bereichs)
        this.addObject(this.peetie, 999999, 999999);

        // UI und Raster erstellen
        createUI();
        // Peeties Ablauf organisieren (kann bei Bedarf auskommentiert werden)
    }

    public void started() {
        // Alle Actor von der Welt entfernen
        List<Actor> actors = (List<Actor>)this.getObjects(Actor.class);
        for(Actor a : actors) {
            this.removeObject(a);
        }

        // Welt neu aufbauen
        initializeWorld();

        // Objekte in der Welt verteilen
        SzenarioKonfigurator.objekteVerteilen(this.aktuellesSzenario, this);
        if(this.aktuellesSzenarioString.equals("e7m5")) {
            Sound.play("BGMUSIC");
        }

        // Peeties Ablauf organisieren (kann bei Bedarf auskommentiert werden)
        this.peetie._runOrganize(); 
    }

    public void stopped() {

        if(this.aktuellesSzenarioString.equals("e7m5")) {
            Sound.stop("BGMUSIC");
        }

    }

    /**
     * Raster auf der Welt
     */
    public void createGrid()
    {
        this.gfImageBG.setColor(this.rasterfarbe);
        int x = 0;
        while (x <= this.getWidth()) {
            this.gfImageBG.drawLine(x, 0, x, this.getHeight());
            x = x + this.gridSize;
        }
        int y = 0;
        while (y <= this.getHeight()) {
            this.gfImageBG.drawLine(0, y, this.getWidth(), y);
            y = y + this.gridSize;
        }
    }

    public void createUI() {
        // Alle Show-Text-Texte löschen
        this.alleShowTextTexteLoeschen();

        // Buttons anzeigen
        this.buttonsZeigen();

        // Diverses (z.B. Logo) hinzufügen
        Label logo = new Label();
        GreenfootImage g = new GreenfootImage("efm-logo-150x49.png");
        g.setTransparency(80);
        logo.setImage(g);
        this.addObject(logo, 90,30);

        // Raster erstellen
        if(this.szenarioArt.equals("1-statisch")) { // nur wenn statisches Szenario
            this.createGrid();
        }
    }

    public void buttonsZeigen() { 
        // Name, Shortcut, Standardbild, Hoverbild, Inaktives Bild
        Button btRun = new Button("run", "r", "bt_start.png", "bt_start_clicked.png", null);
        this.addObject(btRun, 25, 575);

        Button btSound = new Button("ton", "t", "bt_soundON.png", "bt_soundON_clicked.png", "bt_soundOFF_inactive.png");
        this.addObject(btSound, 65, 575);

        Button btAnleitung = new Button("help", "h", "bt_help.png", "bt_help_clicked.png", null);
        this.addObject(btAnleitung, 105, 575);
    }

    public void buttonsVerstecken() { 
        List<Button> buttonListe = (List<Button>)this.getObjects(Button.class);
        for(Button einButton : buttonListe) { 
            this.removeObject(einButton); 
        }
    }

    public void alleShowTextTexteLoeschen() {
        for(int x = 1; x <= this.getWidth(); x++) {
            for(int y = 1; y <= this.getHeight(); y++) {
                this.showText("", x, y);
            }
        }
    }

    /**
     * Ballerszenarien
     */
    public void schrottSteuerung() {
        if(Greenfoot.getRandomNumber(100) > 95) {
            this.addObject(new Schrott(), Greenfoot.getRandomNumber(this.getWidth() - 50) + 50, 30);
        }
    }

    public void asteroidenSteuerung() {
        this.neuenAsteroidenAufWeltSetzen();
        if(((List<Gegner>)this.getObjects(Gegner.class)).size() <= 0 && this.alleAsteroidenSindSchonDa) {
            this.showText("Alle Asteroiden zerstört - und ab zu Mutter Erde!!!", this.getWidth()/2, this.getHeight()/2);
            while(this.peetie.getY() > 20) {
                this.peetie.setLocation(this.peetie.getX(), this.peetie.getY() - 7);
                Greenfoot.delay(3);
            }
            Greenfoot.stop();
        }
    }

    public void drohnenSteuerung() {
        this.drohnenCounter--;
        if(this.drohnenCounter <= 0) {
            if(this.gegnertyp.equals("kampfdrohnen")) {
                this.neueKampfDrohneAufWeltSetzen(); 
            } else if(this.gegnertyp.equals("scoutdrohnen")) {
                this.neueScoutdrohneAufWeltSetzen();
            }
            this.drohnenCounter = this.drohnenVerzoegerung;
        }
        if(((List<Gegner>)this.getObjects(Gegner.class)).size() <= 0 && this.alleDrohnenSindSchonDa) {
            this.showText("Alle Drohnen zerstört - sehr gut!!!", this.getWidth()/2, this.getHeight()/2);
            Greenfoot.stop();
        }

    }

    public void objektAbschussRegistrieren() {
        this.peetie._setObjectShotDown(true);
    }

    public void neuenAsteroidenAufWeltSetzen() {
        if(((List<Gegner>)this.getObjects(Gegner.class)).size() < this.anzahlAsteroiden && !this.alleAsteroidenSindSchonDa) {
            Asteroid a = new Asteroid();
            this.addObject(a, 70, 40);
        } else {
            this.alleAsteroidenSindSchonDa = true;
        }
    }

    public void neueScoutdrohneAufWeltSetzen() {
        if(((List<Gegner>)this.getObjects(Gegner.class)).size() <= this.maxAnzahlDrohnen && !this.alleDrohnenSindSchonDa) {
            Scoutdrohne s = new Scoutdrohne();
            this.addObject(s, 70, 40);
        } else {
            this.alleDrohnenSindSchonDa = true;
        }
    }

    public void neueKampfDrohneAufWeltSetzen() {
        if(((List<Gegner>)this.getObjects(Gegner.class)).size() <= this.maxAnzahlDrohnen && !this.alleDrohnenSindSchonDa) {
            Kampfdrohne k = new Kampfdrohne();
            this.addObject(k, 50, 50);
        } else {
            this.alleDrohnenSindSchonDa = true;
        }
    }
    // Getter und Setter
    public int getGridSize() {
        return gridSize;
    }

    public Peetie getPeetie() {
        return this.peetie;
    }

    public SzenarioTyp getAktuellesSzenario() {
        return this.aktuellesSzenario;
    }
}
