import greenfoot.*;
import java.util.*;

public class Start extends World {
    private List<Label> szenarioLabels = new ArrayList<>();
    private Map<String, Boolean> missionHinzugefuegt = new HashMap<>();
    private Map<String, String> episodenUeberschriften = new HashMap<>();

    public Start() {
        super(960, 540, 1);

        // Überschrift hinzufügen
        GreenfootImage header = new GreenfootImage("MISSIONSAUSWAHL", 40, Color.WHITE, new Color(0, 0, 0, 0));
        getBackground().drawImage(header, (getWidth() - header.getWidth()) / 2, 30);

        // Beschreibungstext hinzufügen
        GreenfootImage beschreibung = new GreenfootImage("Mission durch Klick auswählen. Vorher Run drücken.", 20, Color.WHITE, new Color(0, 0, 0, 0));
        getBackground().drawImage(beschreibung, (getWidth() - beschreibung.getWidth()) / 2, 80);

        // Schriftgröße und Positionen definieren
        int fontSize = 14;
        int headerFontSize = 18;
        int yOffset = fontSize + 14;
        int xOffset = 200;

        // Positionen für die Spalten
        int xStartBase = 150;
        int currentX = xStartBase;
        int currentY = 150;

        // Anpassung der Y-Position für die erste Spalte
        int firstColumnAdjustment = -15;
        boolean isFirstColumn = true;

        // Map für Episoden und Überschriften initialisieren
        episodenUeberschriften.put("e1", "ANWEISUNGEN");
        episodenUeberschriften.put("e2", "ZÄHLERSCHLEIFE");
        episodenUeberschriften.put("e3", "VERZWEIGUNGEN");
        episodenUeberschriften.put("e4", "VARIABLEN");
        episodenUeberschriften.put("e5", "WHILE-SCHLEIFE");
        episodenUeberschriften.put("e6", "EIN-/AUSGABE");
        episodenUeberschriften.put("e7", "UNTERPROGRAMME");

        // Boolean-Map initialisieren
        for (String key : episodenUeberschriften.keySet()) {
            missionHinzugefuegt.put(key, false);
        }

        // Alle Enum-Namen durchgehen und sie klickbar machen
        for (SzenarioTyp szenario : SzenarioTyp.values()) {
            String episode = szenario.name().substring(0, 2);  // Holt das Präfix wie "e1", "e2", etc.

            // Justiere die Y-Position nur für die erste Spalte
            if (isFirstColumn) {
                currentY += firstColumnAdjustment;
                isFirstColumn = false;  // Ab der zweiten Spalte nicht mehr justieren
            }

            // Überprüfen, ob genug Platz für zwei Missionen nach der Überschrift bleibt
            if (currentY + 2 * yOffset > getHeight()) {
                currentY = 150;
                currentX += xOffset;
            }

            // Prüfen, ob die Überschrift bereits hinzugefügt wurde
            if (!missionHinzugefuegt.get(episode)) {
                String ueberschrift = episodenUeberschriften.get(episode);
GreenfootImage ueberschriftImage = new GreenfootImage(ueberschrift, headerFontSize, new Color(255, 140, 105), new Color(0, 0, 0, 0));
                
                currentY -= 30; // abstand zwischen letzter mission einer episode und nächster überschrift justieren
                // Fügen Sie die Überschrift 80 Pixel weiter links und 30 Pixel weiter unten ein
                getBackground().drawImage(ueberschriftImage, currentX - 77, currentY + 30);
                currentY += yOffset + 37; // abstand zwischen überschrift und erster mission einer episode justieren
                missionHinzugefuegt.put(episode, true);
            }

            String t = SzenarioKonfigurator.getTitel(szenario);
            String missionNum = szenario.name().substring(3);
            String formattedName = String.format("%s) %s", missionNum, t);

            // Erstellen des GreenfootImage für die Missionen
            GreenfootImage textImage = new GreenfootImage(200, fontSize + 10);
            textImage.setFont(new Font("Monospaced", fontSize));
            textImage.setColor(Color.WHITE);
            textImage.drawString(formattedName, 0, fontSize);

            Label szenarioLabel = new Label(formattedName, szenario, fontSize);
            szenarioLabel.setImage(textImage);
            szenarioLabels.add(szenarioLabel);
            addObject(szenarioLabel, currentX, currentY);

            // Y-Position für die nächste Mission erhöhen
            currentY += yOffset;

            // Wenn das Ende des Bildschirms erreicht ist, neue Spalte beginnen
            if (currentY + yOffset > getHeight()) {
                currentY = 150;
                currentX += xOffset;
            }
        }
    }

    public void act() {
        for (Label label : szenarioLabels) {
            if (Greenfoot.mouseClicked(label)) {
                SzenarioTyp szenario = label.getSzenarioTyp();
                SzenarioKonfigurator.initialisiereSzenario(szenario);
                Mars mars = new Mars(szenario);
                Greenfoot.setWorld(mars);
            }
        }
    }
}
